/*
 * Programming in Java, Jacobs University, 2015
 *
 * This file implements a simple 'bound property' of a bean, a
 * reusable software component that follows certain coding
 * conventions.  A bound property allows other pieces of code to
 * listen for property changes (see the listener/observer design
 * pattern).
 */ 

import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;

public class BoundBean
    implements java.io.Serializable
{
    /*
     * A version number that needs to be bumped if there are
     * incompabible changes in the bean implementation.
     */

    private static final long serialVersionUID = 1L;

    /*
     * A bound property 'price' together with the data structures
     * necessary to support property change listeners.
     */

    private PropertyChangeSupport pcs;
    public static final String PRICE = "price";
    private int price;

    /*
     * A bean class must have a public default constructor, it may
     * have additional constructors.
     */

    public BoundBean()
    {
        pcs = new PropertyChangeSupport(this);
    }

    public BoundBean(int price)
    {
	this();
	this.price = price;
    }

    public void addPropertyChangeListener(PropertyChangeListener pcl)
    {
        pcs.addPropertyChangeListener(pcl);
    }

    public void removePropertyChangeListener(PropertyChangeListener pcl)
    {
        pcs.removePropertyChangeListener(pcl);
    }
    
    public void setPrice(int newValue)
    {
        if (newValue != price) {
            int oldValue = price;
            price = newValue;
            pcs.firePropertyChange(PRICE,
                                   new Integer(oldValue),
                                   new Integer(newValue));
        }
    }

    public int getPrice()
    {
        return price;
    }

    public static void main(String[] args)
    {
	BoundBean b = new BoundBean(42);
	
	b.addPropertyChangeListener(e -> {
		if (e.getPropertyName() == BoundBean.PRICE) {
		    System.out.println("price change " + e.getOldValue()
				       + " -> " + e.getNewValue());
		}
	    });

	b.setPrice(b.getPrice());
	b.setPrice(b.getPrice() + 2);
	b.setPrice(b.getPrice() - 48);
    }
}
